package main

import (
	"errors"
	"fmt"
	"os"
	"os/exec"
)

func createUserFiles(username string) {
	// create user directories
	userPath := fmt.Sprintf("%s/user/%s/ttyrec/", config.NethackLauncher.HackDir, username)
	exec.Command("mkdir", "-p", userPath).Run()

	// copy in rc file if it does not exist
	hackRCLoc := fmt.Sprintf("%s/.nethackrc", config.NethackLauncher.HackDir)
	if _, err := os.Stat(fmt.Sprintf("%s/user/%s/.nethackrc", config.NethackLauncher.HackDir, username)); errors.Is(err, os.ErrNotExist) {
		println(fmt.Sprintf("%s/user/%s/.nethackrc does not exist, copying it.", config.NethackLauncher.HackDir, username))
		hackRCDest := fmt.Sprintf("%s/user/%s/.nethackrc", config.NethackLauncher.HackDir, username)
		exec.Command("cp", hackRCLoc, hackRCDest).Run()
	} else {
		println(fmt.Sprintf("%s/.nethackrc exists, not overwriting.", config.NethackLauncher.HackDir))
	}

	// create necessary directories if they dont exist
	userpath := fmt.Sprintf("%s/user/%s", config.NethackLauncher.HackDir, username)

	if _, err := os.Stat(fmt.Sprintf("%s/dumps/", userpath)); os.IsNotExist(err) {
		os.Mkdir(fmt.Sprintf("%s/dumps/", userpath), os.ModeDir)
	}
	if _, err := os.Stat(fmt.Sprintf("%s/save/", userpath)); os.IsNotExist(err) {
		os.Mkdir(fmt.Sprintf("%s/save/", userpath), os.ModeDir)
	}

	// create necessary files if they dont exist
	os.OpenFile(fmt.Sprintf("%s/logfile", userpath), os.O_RDONLY|os.O_CREATE, 0666)
	os.OpenFile(fmt.Sprintf("%s/perm", userpath), os.O_RDONLY|os.O_CREATE, 0666)
	os.OpenFile(fmt.Sprintf("%s/xlogfile", userpath), os.O_RDONLY|os.O_CREATE, 0666)

	// create record symlink
	exec.Command("ln", "-s", fmt.Sprintf("%s/record", config.NethackLauncher.HackDir), fmt.Sprintf("%s/record", userpath)).Run()

	// create sysconf symlink
	exec.Command("ln", "-s", fmt.Sprintf("%s/sysconf", config.NethackLauncher.HackDir), fmt.Sprintf("%s/sysconf", userpath)).Run()

	// move in nhdat file if it does not exist
	exec.Command("cp", fmt.Sprintf("%s/nhdat", config.NethackLauncher.HackDir), userpath).Run()

	// run sotw prep script
	if _, err := os.Stat(fmt.Sprintf("%s/sotw/", userpath)); os.IsNotExist(err) {
		println("running /bin/link_sotw.sh")
		out, err := exec.Command("/bin/link_sotw.sh", userpath, config.NethackLauncher.HackDir).Output()
		if err != nil {
			println("ope: ")
			fmt.Sprintf("%s", err)
		}
		println(string(out))
	}
}
